function drawEfficiency(precise, value, effTreshold) {
    var canvas = document.getElementById("efficiencyCanvas");
    var ctx = canvas.getContext("2d");
    ctx.clearRect(0, 0, canvas.width, canvas.height);

    var pickerBodyHeight = Math.round(canvas.height * 0.15);
    var outlineWidth = 1;
    var barHeight = canvas.height - pickerBodyHeight;
    var barWidth = canvas.width - 2 * outlineWidth;
    var barX0 = outlineWidth;

    var fontColor = "#000000";
    var outlineColor = "#000000";
    var bkColor = "#FFFFFF";
    var badEfficiencyColor = "#D0D0D0";
    var goodEfficiencyColor = "#FFA500";

    ctx.beginPath();
    ctx.fillStyle = outlineColor;
    ctx.rect(0, 0, barWidth + 2 * outlineWidth, barHeight + 2 * outlineWidth);
    ctx.fill();
    ctx.closePath();

    ctx.beginPath();
    ctx.fillStyle = bkColor;
    ctx.rect(outlineWidth, outlineWidth, barWidth, barHeight);
    ctx.fill();
    ctx.closePath();

    var valuePixels = Math.round(barWidth / 100. * value);
    var effTresholdPixels = Math.round(barWidth / 100. * effTreshold);

    var goodEfficiency = false;

    var solidFillGreyLeft = barX0;
    var solidFillGreyRight = barX0;

    var solidFillOrangeLeft = barX0;
    var solidFillOrangeRight = barX0;

    if (value <= effTreshold) {
        goodEfficiency = false;
        solidFillGreyLeft = barX0;
        solidFillGreyRight = solidFillGreyLeft + valuePixels;
    }
    else if (value > effTreshold) {
        goodEfficiency = true;
        solidFillGreyLeft = barX0;
        solidFillGreyRight = solidFillGreyLeft + effTresholdPixels;
        solidFillOrangeLeft = solidFillGreyRight;
        solidFillOrangeRight = solidFillOrangeLeft + (valuePixels - effTresholdPixels - 1);
    }
    var solidFillLeft = barX0;
    var solidFillRight = barX0;
    var gradientFillLeft = barX0;
    var gradientFillRight = barX0;

    var effFillColor = badEfficiencyColor;
    if (goodEfficiency) {
        effFillColor = goodEfficiencyColor;
    }

    if (!precise) {
        var gradientFillPercent = 10.;
        var gradientPixels = Math.round(barWidth / 100. * gradientFillPercent);
        if (solidFillGreyRight - solidFillGreyLeft > 0) {
            solidFillLeft = solidFillGreyLeft;
            solidFillRight = solidFillGreyRight;
        }
        if (solidFillOrangeRight - solidFillOrangeLeft > 0) {
            solidFillLeft = solidFillOrangeLeft;
            solidFillRight = solidFillOrangeRight;
        }
        gradientFillLeft = solidFillLeft;
        gradientFillRight = solidFillRight;
        if (solidFillRight - solidFillLeft < gradientPixels) {
            solidFillRight = solidFillLeft;
        }
        else {
            solidFillRight = solidFillRight - gradientPixels;
            gradientFillLeft = solidFillRight;
        }
    }

    if (solidFillGreyRight - solidFillGreyLeft > 0) {
        ctx.fillStyle = effFillColor;
        ctx.fillRect(solidFillGreyLeft, outlineWidth, solidFillGreyRight - solidFillGreyLeft, barHeight);
    }
    if (solidFillOrangeRight - solidFillOrangeLeft > 0) {
        ctx.fillStyle = effFillColor;
        ctx.fillRect(solidFillOrangeLeft, outlineWidth, solidFillOrangeRight - solidFillOrangeLeft, barHeight);
    }
    if (gradientFillRight - gradientFillLeft > 0) {
        var grd = ctx.createLinearGradient(gradientFillLeft, 0, gradientFillRight, 0);
        grd.addColorStop(0, effFillColor);
        grd.addColorStop(1, bkColor);
        ctx.fillStyle = grd;
        ctx.fillRect(gradientFillLeft, outlineWidth, gradientFillRight - gradientFillLeft, barHeight);
    }

    var pickerWidth = 4;
    var pickerRoofHeight = 4;
    var pickerY0 = canvas.height - pickerBodyHeight - pickerRoofHeight;
    var pickerX0 = effTresholdPixels;

    ctx.fillStyle = outlineColor;
    ctx.beginPath();
    ctx.moveTo(pickerX0, pickerY0);
    ctx.lineTo(pickerX0 + pickerWidth, pickerY0 + pickerRoofHeight);
    ctx.lineTo(pickerX0 + pickerWidth, pickerY0 + (pickerRoofHeight + pickerBodyHeight));
    ctx.lineTo(pickerX0 - pickerWidth, pickerY0 + (pickerRoofHeight + pickerBodyHeight));
    ctx.lineTo(pickerX0 - pickerWidth, pickerY0 + pickerRoofHeight);
    ctx.lineTo(pickerX0, pickerY0);
    ctx.closePath();
    ctx.fill();
}