function render(viewData) {
    if (!Page.get().isTemplate())
        Page.get().log(JSON.stringify(viewData, null, '\t'));

    renderLoopSummary(viewData.loopSummary);
    renderTraitsSummary(viewData.traitsSummary);
    renderTripCountSummary(viewData.tripCountSummary);
    renderEfficiency(viewData.efficiencySummary);
    renderInstrMix(viewData.instructionMix);
    renderModeling(viewData.modeling);
}

function renderInstrMix(data) {
    var el = document.getElementById('instr_mix');
    if (el == null) return;

    el.Visible(data !== undefined);
    if (data === undefined) return;

    var element = document.createElement("div");
    var button = document.createElement("span");
    button.innerHTML = "Reverse";
    // TODO: create new request - instruction mix request
    button.onclick = function () { LoopRequest.send(render); };
    //element.appendChild(button);
    data.summary.forEach(function (item) {
        var name = document.createElement("span");
        name.classList.add('name');
        name.innerHTML = item.displayName + ': ';
        element.appendChild(name);

        var value = document.createElement("span");
        value.classList.add('value');
        value.innerHTML = item.value;
        element.appendChild(value);
    });
    var root = document.getElementById("instr_mix_summary");
    while (root.firstChild != null) root.removeChild(root.firstChild);
    root.appendChild(element);

    renderTextHierarchy(data.details, 'instr_mix_details_text', null,
        function (o) {
            return parseFloat(o.value).round(0) + '% (' + o.number + ')';
        });

    renderAreaHierarchy(data.details, 'instr_mix_details_area');

    el.classList.remove('loading');
}

function renderModeling(data) {
    var el = document.getElementById('modeling');
    if (el == null) return;

    el.Visible(data !== undefined);
    if (data === undefined) return;

    el.classList.remove('loading');
}

function renderLoopSummary(data) {
    var el = document.getElementById('loop_summary');
    if (el == null) return;

    el.Visible(data !== undefined);
    if (data === undefined) return;

    el.querySelector("#loop_type").innerHTML = data.loopType.displayName;
    
    var fullpath = el.querySelector("#loop_type_img").src;
    var path = fullpath.substring(0, fullpath.lastIndexOf('/'));
    var imageName = "cycle-48x48.png";
    if (data.loopType.type == "vectorized"){
        imageName = "cycle-vect-48x48.png";
    }
    else if(data.loopType.type == "not_executed"){
        imageName = "cycle-fake-48x48";
    }

    el.querySelector("#loop_type_img").src = path + "/" + imageName;
    el.querySelector("#total_time").innerHTML = data.loopTime.total.round(3) + "s";
    el.querySelector("#self_time").innerHTML = data.loopTime.self.round(3) + "s";

    el.querySelector("#loop_isa").Visible(data.loopISA !== undefined);
    el.querySelector("#loop_isa_descr").Visible(data.loopISA !== undefined);
    if (data.loopISA !==undefined) {
        el.querySelector("#loop_isa").innerHTML = data.loopISA.value;
    }

    el.classList.remove('loading');
}

function renderTraitsSummary(data) {
    var el = document.getElementById('traits_summary');
    if (el == null) return;

    el.Visible(data !== undefined);
    if (data === undefined) return;

    var title = el.querySelector("[name = title]");
    title.classList.remove('with_warning');

    var traits = el.querySelector("#traits");
    traits.innerHTML = data.traits.join(', ');

    var details = el.querySelector("#traits_details");
    while (details.firstChild != null) details.removeChild(details.firstChild);

    if (data.traits !== undefined) {
        data.traits.forEach(function (item) {
            var i = document.createElement('div');
            i.classList.add('trait');
            var trait_name = document.createElement('span');
            trait_name.innerHTML = item;
            i.appendChild(trait_name);
            if (data.tips !== undefined) {
                data.tips.forEach(function (w) {
                    if (w.first == item) {
                        title.classList.add('with_warning');

                        var existed = i.querySelector('.warning');
                        if (existed !== null) {
                            var list = existed.querySelector('ul');
                            var li = document.createElement("li");
                            li.innerHTML = w.second;
                            list.appendChild(li);
                            existed.appendChild(list);
                        } else {
                            trait_name.classList.add('with_warnings');
                            var ii = document.createElement('div');
                            var list = document.createElement("ul");
                            var li = document.createElement("li");
                            li.innerHTML = w.second;
                            list.appendChild(li);
                            ii.appendChild(list);
                            ii.classList.add('warning');
                            i.addEventListener('click', function () {
                                trait_name.classList.toggle('shown');
                                if (ii.offsetHeight != 0) {
                                    ii.style.height = 0;
                                } else {
                                    ii.style.height = ii.calculateHeight() + 'px';
                                }
                            });
                            i.appendChild(ii);

                        }
                    }
                });

            }
            details.appendChild(i)
        });
    }

    el.classList.remove('loading');
}

function renderTripCountSummary(data) {
    var el = document.getElementById('trip_count_summary');
    if (el == null) return;

    el.Visible(data !== undefined);
    if (data === undefined) return;

    var tripCount = el.querySelector("#trip_count");
    tripCount.innerHTML = data.tripcount;

    el.classList.remove('loading');
}

function renderEfficiency(data) {
    var el = document.getElementById('efficiency_summary');
    if (el == null)
        return;

    el.Visible(data !== undefined);
    if (data === undefined) return;

    var prefix = '~';
    if (data.confidence) {
        prefix = '';
    }

    var efficiency = data.efficiency;
    if (efficiency > 100.) {
        efficiency = 100.;
        prefix = '>=';
    }

    [].slice.call(el.querySelectorAll('[name=vector_gain]')).forEach(function (e) { e.innerHTML = data.gain + 'x'; });
    [].slice.call(el.querySelectorAll('[name=vector_efficiency]')).forEach(function (e) { e.innerHTML = prefix + efficiency.round(0) + "%"; });
    [].slice.call(el.querySelectorAll('[name=vector_length]')).forEach(function (e) { e.innerHTML = data.vl; });
    [].slice.call(el.querySelectorAll('[name=ref_efficiency_value]')).forEach(function (e) { e.innerHTML = data.referenceEfficiency.round(0) + "% "; });

    var goodEfficiency = efficiency > data.referenceEfficiency;
    el.querySelector("#eff_tooltip_info_orange").Visible(goodEfficiency);
    el.querySelector("#eff_tooltip_info_grey").Visible(!goodEfficiency);
    el.querySelector("#eff_tooltip_warn_approx").Visible(!data.confidence && !(data.efficiency > 100.));
    el.querySelector("#eff_tooltip_warn_too_high_eff").Visible(data.efficiency > 100.);

    drawEfficiency(data.confidence, efficiency, data.referenceEfficiency);

    el.classList.remove('loading');
}
