function clearContent(content) {
    while (content.firstChild !== null) content.removeChild(content.firstChild);
    removeHierarchyEventsProcessing(content);
    var tooltips = document.getElementsByClassName('idvc_popup');
    while (tooltips.length > 0) tooltips[0].parentNode.removeChild(tooltips[0]);
}

function renderTextHierarchy(data, id, displayname, displayvalue, barwidth, collapsedByDefault) {
    var content = document.getElementById(id);
    if (content === null) return;
    clearContent(content);

    var displayname = displayname || function (obj) { return obj.displayName; };
    var displayvalue = displayvalue || function (obj) { return parseFloat(obj.value).round(2) + '%'; };
    var barwidth = barwidth || function (obj) { return obj.value + '%' };
    var collapsedByDefault = collapsedByDefault || true;

    buildHierarchy(content, data, function (html, obj) {
        if (!html || !html.expand || !obj) return;

        var expand = html.expand;

        if (!collapsedByDefault && obj.expanded) expand.classList.add(HierarchyExpanded);
        if (obj.className) expand.classList.add(obj.className);

        var container = document.createElement('div');
        container.className = 'idvc_hierarchy_item';

        var title = document.createElement('div');
        title.className = 'idvc_hierarchy_title';
        title.innerHTML = displayname(obj);

        var value = document.createElement('div');
        value.className = 'idvc_hierarchy_value';

        var text = document.createElement('div');
        text.className = 'idvc_hierarchy_value_text';
        text.innerHTML = displayvalue(obj);

        var barHolder = document.createElement('div');
        barHolder.className = 'idvc_hierarchy_value_bar_container';

        var bar = document.createElement('div');
        bar.className = 'idvc_hierarchy_value_bar';
        bar.style.width = barwidth(obj);

        barHolder.appendChild(bar);
        value.appendChild(text);
        value.appendChild(barHolder);

        container.appendChild(title);
        container.appendChild(value);

        expand.appendChild(container);
    });

    addHierarchyEventsProcessing(content);
}

function renderAreaHierarchy(data, id, decorator) {
    var content = document.getElementById(id);
    if (content === null) return;
    clearContent(content);

    var decorator = decorator || function (obj) { return obj.displayName + ': ' + parseFloat(obj.value).round(2) + '%'; };

    buildHierarchy(content, data, function (html, obj, parentObj) {
        if (!html || !html.expand || !obj) return;
        var expand = html.expand;

        if (obj.expanded) expand.classList.add(HierarchyExpanded);
        if (obj.className) expand.classList.add(obj.className);
        expand.innerHTML = decorator(obj);

        var container = html.container;
        var root = container ? container : expand;
        var width = (parentObj && parentObj.value) ? obj.value / parentObj.value * 100 : 100;
        if (obj.value > 1) {
            root.style.width = width + '%';
        } else {
            root.style.display = 'none';
        }

        var color = Palette.Default(obj.value / 100.0);
        root.style.backgroundColor = color;
        expand.style.backgroundColor = color;
        if (parentObj && parentObj.value == 100) {
            root.style.borderColor = '#fff';
            root.style.borderWidth = '1px';
        } else if (root.classList.contains(HierarchyLeafClass)) {
            root.style.border = '1px solid #fff';
            if (root.previousSibling != null)
                root.style.borderLeft = '0px';
        } else
            root.style.borderColor = color;
    });

    addHierarchyEventsProcessing(content, undefined, function (elem) {
        function getParentContainer(child) {
            var result;

            while (child) {
                if (child.classList &&
                    child.classList.contains('idvc_hierarchy_container')) {
                    result = child;
                    break;
                }
                child = child.parentNode;
            }

            return result;
        }

        function fitChildren(parent) {
            if (parent) {
                var height = 0;
                [].forEach.call(parent.children, function (child) {
                    child.style.height = 'auto';
                    if (child.offsetHeight > height) {
                        height = child.offsetHeight;
                    }
                });
                [].forEach.call(parent.children, function (child) {
                    child.style.height = height + 'px';
                });
            }
        }

        var container = getParentContainer(elem);
        var parent;
        while (container) {
            parent = container.parentNode;
            fitChildren(parent);
            container = getParentContainer(parent);
        }
    });

    addExpansion(content, function (elem) {
        if (elem && elem.classList) {
            return elem.classList.contains('idvc_hierarchy_caption') ||
              elem.classList.contains('idvc_hierarchy_leaf');
        }
        return false;
    });
}

var colorPicker = (function(){
  var startHue = 0;
  var step = 77;

  var hue = startHue;
  return {
    next: function() {
      var color = 'hsl(' + hue + ', 100%, 35%)';
      hue += step;
      return color;
    },
    rewind: function() {
      hue = startHue;
    }
  }
})();

function DrawPalette() {
    var draw = function (canvas, color, step) {
        var step = step || 1;
        var color = color || function (x) { return Palette.Default(x); }
        var canvas = document.getElementById(canvas);
        var ctx = canvas.getContext("2d");
        var w = canvas.getAttribute("width");
        var h = canvas.getAttribute("height");
        var l = 0.91 * w;
        for (var i = 0; i < l; i+=step) {
            ctx.beginPath();
            ctx.fillStyle = color(i / l);
            ctx.rect(i, 0, step, h - 10);
            ctx.fill();
            if (i % parseInt(l / 20) == 0) {
                ctx.fillText(parseInt(i / (l / 100) + 1).toString(), i, h);
            }
            ctx.closePath();
        }
    }
    draw("original", colorPicker.next,10);
    draw("cold", Palette.Cold);
    draw("soft_blue", Palette.SoftBlue);
    draw("striped", Palette.Striped);
}

var Palette = (function () {
    var hsl = function (h, s, l) {
        return 'hsl(' + h + ',' + s + '%,' + l + '%)';
    };
    var linear = function (x, a, b, round) {
        var round = round || function (x) { return x; };
        return round(a + (b - a) * x);
    };
    var period = function (x, a, b, c) {
        var m = (a + b) * 0.5;
        var p = (b - a) * 0.5;
        var a = -Math.PI / 2 + Math.PI * x;
        return m + p * Math.sin(2 * c * a);
    };
    var toLog = function (x) { return Math.log(100 * x) / Math.log(100); };
    return {
        Cold: function (x) {
            return hsl(linear(x, 200, 230, parseInt), linear(x, 30, 100), linear(x, 70, 30));
        },
        Striped: function (x) {
            return hsl(linear(x, 190, 220, parseInt), linear(x, 50, 90), period(toLog(x), 25, 75, 2));
        },
        SoftBlue: function (x) {
            return hsl(linear(x, 190, 256, parseInt), linear(x, 30, 100), linear(x, 55, 15));
        },
        Default: function (x) {
            return this.Striped(x);
        }
    }
})();

var HierarchyContainerClass = 'idvc_hierarchy_container';
var HierarchyCaptionClass = 'idvc_hierarchy_caption';
var HierarchyLeafClass = 'idvc_hierarchy_leaf';
var HierarchyBodyClass = 'idvc_hierarchy_body';
var HierarchyExpanded = 'idvc_hierarchy_expanded';


var hierarchyEvents = {};
function removeHierarchyEventsProcessing(parent) {
    if (parent in hierarchyEvents) {
        var clicks = hierarchyEvents[parent];
        for (var i = clicks.length; i--;) {
            var click = clicks[i];
            parent.removeEventListener('click', click, false);
        }
    }
}

function addHierarchyEventsProcessing(parent, beforeExpand, afterExpand) {
    beforeExpand = beforeExpand || function () { return false; };
    afterExpand = afterExpand || function () { };
    parent = parent || document;

    if (!(parent in hierarchyEvents)) {
        hierarchyEvents[parent] = [];
    }

    var click = function (e) {
        e = e || window.event;
        var element = e.target;

        while (element &&
         element.classList &&
         !element.classList.contains(HierarchyCaptionClass)) {
            element = element.parentNode;
        }

        if (element &&
            element.classList &&
            element.classList.contains(HierarchyCaptionClass) &&
            !beforeExpand(element, element.classList.contains(HierarchyExpanded))) {
            element.classList.toggle(HierarchyExpanded);
            afterExpand(element);
        }
    };

    hierarchyEvents[parent].push(click);
    parent.addEventListener('click', click, false);
}

function addHierarchyHTML(parent, hasChildren) {
  var expandItem = document.createElement('div');

  var body, container, root;

  if (hasChildren) {
    expandItem.className = HierarchyCaptionClass;

    container = document.createElement('div');
    container.className = HierarchyContainerClass;
    container.appendChild(expandItem);

    body = document.createElement('div');
    body.className = HierarchyBodyClass;
    container.appendChild(body);

    root = container;
  } else {
    expandItem.className = HierarchyLeafClass;

    root = expandItem;
  }

  parent.appendChild(root);

  return { expand: expandItem, body: body, container: container };
}

function buildHierarchyLevel(parent, parentObj, processHTML) {
  if (!parent || !parentObj) return;

  var objs = parentObj.children;
  if (!Array.isArray(objs)) return;

  objs.forEach(function(obj) {
    var html = addHierarchyHTML(parent, obj.children && obj.children.length);
    processHTML(html, obj, parentObj);

    buildHierarchyLevel(html.body, obj, processHTML);
  });

}

function buildHierarchy(parent, obj, processHTML) {
  processHTML = processHTML || function(html, obj){
    if (!html || !html.expand || !obj) return;

    var expand = html.expand;

    if (obj.displayName) expand.innerHTML = obj.displayName;
    if (obj.expanded) expand.classList.add(HierarchyExpanded);
    if (obj.className) expand.classList.add(obj.className);
  };

  if (Array.isArray(obj)) {
    obj = { children: obj, value: 100 };
  }

  buildHierarchyLevel(parent, obj, processHTML);
}

function getElementPos(elem) {
  var rect = elem.getBoundingClientRect();

  var docElement = document.documentElement;
  var scrollTop = window.pageYOffset || docElement.scrollTop;
  var scrollLeft = window.pageXOffset || docElement.scrollLeft;

  var top  = rect.top + scrollTop - docElement.clientTop;
  var left = rect.left + scrollLeft - docElement.clientLeft;

  return {x: left, y: top};
}

var setFont = function(elem, styleSet) {
  if (elem && styleSet) {
    elem.style.fontFamily = styleSet.fontFamily;
    elem.style.fontSize = styleSet.fontSize;
    elem.style.fontStyle = styleSet.fontStyle;
    elem.style.fontWeight = styleSet.fontWeight;
    elem.style.fontVariant = styleSet.fontVariant;
  }
};

var setBorser = function(elem, styleSet) {
  if (elem && styleSet) {
    var sides = ['Left', 'Top', 'Right', 'Bottom'];
    var items = ['Width', 'Color', 'Style'];
    sides.forEach(function(side) {
      items.forEach(function(item) {
        var name = 'border' + side + item;
        elem.style[name] = styleSet[name];
      })
    });
  }
};

function addExpansion(parent, acceptCallback, background) {
  var tooltipInfo = {
    tooltip: null,
    timer: null,
    elem: null
  };

  acceptCallback = acceptCallback || function() { return true; }

  var hideTooltip = function() {
    if (tooltipInfo.tooltip) {
      document.body.removeChild(tooltipInfo.tooltip);
    }

    if (tooltipInfo.timer) {
      window.clearTimeout(tooltipInfo.timer);
      tooltipInfo.timer = null;
    }

    tooltipInfo.tooltip = null;
    tooltipInfo.elem = null;
  };

  var showTooltip = function(elem) {
    tooltipInfo.timer = null;

    if (!elem) return;

    var tooltipText = elem.innerHTML;
    if (!tooltipText) {
      return;
    }

    var overPos = getElementPos(elem);
    var overWidth = elem.offsetWidth;
    if (overPos.x + overWidth > window.innerWidth + window.scrollX) {
      overWidth = window.innerWidth + window.scrollX - overPos.x;
    }

    var tooltip = document.createElement('div');
    tooltip.className = 'idvc_popup';
    tooltip.innerHTML = tooltipText;
    var elemStyle = window.getComputedStyle(elem, null);
    setFont(tooltip, elemStyle);
    tooltip.style.color = elemStyle.color;
    tooltip.style.paddingLeft = elemStyle.paddingLeft;
    tooltip.style.paddingTop = elemStyle.paddingTop;
    tooltip.style.paddingRight = elemStyle.paddingRight;
    tooltip.style.paddingBottom = elemStyle.paddingBottom;
    setBorser(tooltip, elemStyle);
    tooltip.style.backgroundColor = background || elemStyle.backgroundColor;

    document.body.appendChild(tooltip);

    if ((tooltip.offsetWidth - 1 > overWidth)) {
      setTooltipPos(overPos.x, overPos.y, tooltip);
      tooltipInfo.tooltip = tooltip;
    } else {
      document.body.removeChild(tooltip);
    }
  };

  var setTooltipPos = function(tooltipX, tooltipY, tooltip) {
    if (tooltip.offsetWidth > 0.8 * window.innerWidth) {
      tooltip.style.height = 'auto';
      tooltip.style.width = Math.floor(0.8 * window.innerWidth) + 'px';
    }

    var tooltipLeft = tooltipX;
    if (tooltipLeft + tooltip.offsetWidth >
        window.innerWidth + window.scrollX) {
      tooltipLeft = window.innerWidth +
        window.scrollX - tooltip.offsetWidth;
    }
    var tooltipTop = tooltipY;
    if (tooltipTop + tooltip.offsetHeight >
        window.innerHeight + window.scrollY) {
      tooltipTop = window.innerHeight +
        window.scrollY - tooltip.offsetHeight;
    }

    tooltip.style.left = tooltipLeft + 'px';
    tooltip.style.top = tooltipTop + 'px';
  };

  parent.onmouseout = hideTooltip;
  parent.onmousemove = function (e) {
      e = e || event;

      var elem = e.target;
      if (elem === tooltipInfo.elem) return;

      hideTooltip();

      if (acceptCallback(elem)) {
          tooltipInfo.elem = elem;
          tooltipInfo.timer = window.setTimeout(showTooltip.bind(undefined, elem), 250);
      }
  };
}
